using System;
using System.Runtime.InteropServices;

namespace ParseAnsi
{
	/// <summary>
	/// Summary description for Win32API.
	/// </summary>
	public class Win32API
	{
		// Declare the SetConsoleCtrlHandler function
		// as external and receiving a delegate.

		[DllImport("Kernel32")]
		public static extern bool SetConsoleCtrlHandler(HandlerRoutine Handler, bool Add);

		// A delegate type to be used as the handler routine
		// for SetConsoleCtrlHandler.
		public delegate bool HandlerRoutine(CtrlTypes CtrlType);

		// An enumerated type for the control messages
		// sent to the handler routine.
		public enum CtrlTypes
		{
			CTRL_C_EVENT = 0,
			CTRL_BREAK_EVENT,
			CTRL_CLOSE_EVENT,
			CTRL_LOGOFF_EVENT = 5,
			CTRL_SHUTDOWN_EVENT
		}

		[StructLayout(LayoutKind.Sequential)]
			public struct COORD
		{
			public short x;
			public short y;
		}

		[StructLayout(LayoutKind.Sequential)]
			public struct SMALL_RECT
		{
			public short Left;
			public short Top;
			public short Right;
			public short Bottom;
		}

		[StructLayout(LayoutKind.Sequential)]
			public struct	CONSOLE_SCREEN_BUFFER_INFO
		{
			public COORD dwSize;
			public COORD dwCursorPosition;
			public CharacterAttributes wAttributes;
			public SMALL_RECT srWindow;
			public COORD dwMaximumWindowSize;
		}

		[DllImport("kernel32.dll", EntryPoint="GetStdHandle", SetLastError=true, CharSet=CharSet.Auto, CallingConvention=CallingConvention.StdCall)]
		public static extern IntPtr GetStdHandle(int nStdHandle);

		[DllImport("kernel32.dll", EntryPoint="SetConsoleTextAttribute", SetLastError=true, CharSet=CharSet.Auto, CallingConvention=CallingConvention.StdCall)]
		public static extern bool SetConsoleTextAttribute(IntPtr hConsoleOutput, CharacterAttributes wAttributes);

		[DllImport("kernel32.dll", EntryPoint="FillConsoleOutputCharacter", SetLastError=true, CharSet=CharSet.Auto, CallingConvention=CallingConvention.StdCall)]
		public static extern int FillConsoleOutputCharacter(IntPtr hConsoleOutput, byte cCharacter, int nLength, COORD dwWriteCoord, ref int lpNumberOfCharsWritten);

		[DllImport("kernel32.dll", EntryPoint="FillConsoleOutputAttribute", SetLastError=true, CharSet=CharSet.Auto, CallingConvention=CallingConvention.StdCall)]
		public static extern int FillConsoleOutputAttribute(IntPtr hConsoleOutput, CharacterAttributes wAttributes, int nLength, COORD dwWriteCoord, ref int lpNumberOfCharsWritten);

		[DllImport("kernel32.dll", EntryPoint="GetConsoleScreenBufferInfo", SetLastError=true, CharSet=CharSet.Auto, CallingConvention=CallingConvention.StdCall)]
		public static extern int GetConsoleScreenBufferInfo(IntPtr hConsoleOutput, ref CONSOLE_SCREEN_BUFFER_INFO lpConsoleScreenBufferInfo);

		[DllImport("kernel32.dll", EntryPoint="SetConsoleCursorPosition", SetLastError=true, CharSet=CharSet.Auto, CallingConvention=CallingConvention.StdCall)]
		public static extern int SetConsoleCursorPosition(IntPtr hConsoleOutput, COORD dwCursorPosition);

		/* This enumeration lists all of the character attributes. You can combine attributes to */
		/* achieve specific effects.*/
		public enum CharacterAttributes
		{
			FOREGROUND_BLUE = 0x0001,
			FOREGROUND_GREEN = 0x0002,
			FOREGROUND_RED = 0x0004,
			FOREGROUND_INTENSITY = 0x0008,
			BACKGROUND_BLUE = 0x0010,
			BACKGROUND_GREEN = 0x0020,
			BACKGROUND_RED = 0x0040,
			BACKGROUND_INTENSITY = 0x0080,
			COMMON_LVB_LEADING_BYTE = 0x0100,
			COMMON_LVB_TRAILING_BYTE = 0x0200,
			COMMON_LVB_GRID_HORIZONTAL = 0x0400,
			COMMON_LVB_GRID_LVERTICAL = 0x0800,
			COMMON_LVB_GRID_RVERTICAL = 0x1000,
			COMMON_LVB_REVERSE_VIDEO = 0x4000,
			COMMON_LVB_UNDERSCORE = 0x8000
		}
	}
}